@@ -1,9 +1,8 @@
+use crate::utils::{flatten, get_tuple_from_vec3};
 use glam::Vec3A;
 use pyo3::{pyclass, pymethods};
 use rl_ball_sym::simulation::ball::Ball;
 
-use crate::utils::get_tuple_from_vec3;
-
 #[pyclass]
 #[derive(Clone, Copy)]
 pub struct ShotType;
@@ -12,6 +11,16 @@ pub struct ShotType;
 impl ShotType {
     #[classattr]
     pub const GROUND: usize = 0;
+    #[classattr]
+    pub const JUMP: usize = 1;
+}
+
+fn get_str_from_shot_type(type_: usize) -> String {
+    match type_ {
+        ShotType::GROUND => String::from("GROUND"),
+        ShotType::JUMP => String::from("JUMP"),
+        _ => unreachable!(),
+    }
 }
 
 #[pyclass]
@@ -26,7 +35,7 @@ pub struct TargetOptions {
 #[pymethods]
 impl TargetOptions {
     #[new]
-    fn __new__(min_slice: Option<usize>, max_slice: Option<usize>, use_absolute_max_values: Option<bool>, all: Option<bool>) -> Self {
+    const fn __new__(min_slice: Option<usize>, max_slice: Option<usize>, use_absolute_max_values: Option<bool>, all: Option<bool>) -> Self {
         Self {
             min_slice,
             max_slice,
@@ -71,7 +80,7 @@ pub struct BasicShotInfo {
     #[pyo3(get)]
     found: bool,
     #[pyo3(get)]
-    time: Option<f32>,
+    time: f32,
     #[pyo3(get)]
     shot_type: Option<usize>,
 }
@@ -80,7 +89,7 @@ impl BasicShotInfo {
     pub const fn not_found() -> Self {
         BasicShotInfo {
             found: false,
-            time: None,
+            time: -1.,
             shot_type: None,
         }
     }
@@ -88,7 +97,7 @@ impl BasicShotInfo {
     pub const fn found(time: f32, shot_type: usize) -> Self {
         BasicShotInfo {
             found: true,
-            time: Some(time),
+            time,
             shot_type: Some(shot_type),
         }
     }
@@ -97,15 +106,15 @@ impl BasicShotInfo {
 #[pymethods]
 impl BasicShotInfo {
     fn __str__(&self) -> String {
-        match self.time {
-            Some(time) => format!("Found at time: {:.2}", time),
+        match self.shot_type {
+            Some(shot_type) => format!("{} found at time: {:.2}", get_str_from_shot_type(shot_type), self.time),
             None => String::from("Not found"),
         }
     }
 
     fn __repr__(&self) -> String {
-        match self.time {
-            Some(time) => format!("BasicShotInfo(found=True, time={})", time),
+        match self.shot_type {
+            Some(shot_type) => format!("BasicShotInfo(found=True, time={}, type={})", self.time, get_str_from_shot_type(shot_type)),
             None => String::from("BasicShotInfo(found=False)"),
         }
     }
@@ -142,17 +151,12 @@ impl BallSlice {
 }
 
 impl BallSlice {
-    pub fn from(ball: &Ball) -> Self {
-        let time = ball.time;
-        let location = get_tuple_from_vec3(ball.location);
-        let velocity = get_tuple_from_vec3(ball.velocity);
-        let angular_velocity = get_tuple_from_vec3(ball.angular_velocity);
-
+    pub const fn from(ball: &Ball) -> Self {
         BallSlice {
-            time,
-            location,
-            velocity,
-            angular_velocity,
+            time: ball.time,
+            location: get_tuple_from_vec3(ball.location),
+            velocity: get_tuple_from_vec3(ball.velocity),
+            angular_velocity: get_tuple_from_vec3(ball.angular_velocity),
         }
     }
 }
@@ -160,16 +164,22 @@ impl BallSlice {
 #[pyclass]
 #[allow(dead_code)]
 pub struct AdvancedShotInfo {
+    #[pyo3(get)]
+    shot_vector: (f32, f32, f32),
     #[pyo3(get)]
     final_target: (f32, f32, f32),
     #[pyo3(get)]
     distance_remaining: f32,
     #[pyo3(get)]
+    required_jump_time: Option<f32>,
+    #[pyo3(get)]
     path_samples: Vec<(f32, f32)>,
+    #[pyo3(get)]
+    current_path_point: (f32, f32, f32),
 }
 
 impl AdvancedShotInfo {
-    pub fn get_distance_remaining(&self) -> f32 {
+    pub const fn get_distance_remaining(&self) -> f32 {
         self.distance_remaining
     }
 }
@@ -177,25 +187,47 @@ impl AdvancedShotInfo {
 #[pymethods]
 impl AdvancedShotInfo {
     fn __str__(&self) -> String {
-        format!("Final target: {:?}, distance remaining: {:.2}", self.final_target, self.distance_remaining)
+        match self.required_jump_time {
+            Some(required_jump_time) => format!(
+                "Shot vector: {:?}, Final target: {:?}, distance remaining: {:.0}, required jump time: {:.1}",
+                self.shot_vector, self.final_target, self.distance_remaining, required_jump_time
+            ),
+            None => format!(
+                "Shot vector: {:?}, Final target: {:?}, distance remaining: {:.0}",
+                self.shot_vector, self.final_target, self.distance_remaining
+            ),
+        }
     }
 
     fn __repr__(&self) -> String {
         format!(
-            "AdvancedShotInfo(final_target={:?}, distance_remaining={}, path_samples=[{} items])",
+            "AdvancedShotInfo(shot_vector={:?}, final_target={:?}, distance_remaining={}, required_jump_time: {:?}, path_samples=[{} items], current_path_point:{:?})",
+            self.shot_vector,
             self.final_target,
             self.distance_remaining,
-            self.path_samples.len()
+            self.required_jump_time,
+            self.path_samples.len(),
+            self.current_path_point,
         )
     }
 }
 
 impl AdvancedShotInfo {
-    pub fn from(target: Vec3A, distance_remaining: f32, path_samples: Vec<(f32, f32)>) -> Self {
+    pub const fn from(
+        shot_vector: Vec3A,
+        target: Vec3A,
+        distance_remaining: f32,
+        path_samples: Vec<(f32, f32)>,
+        required_jump_time: Option<f32>,
+        current_path_point: Vec3A,
+    ) -> Self {
         AdvancedShotInfo {
-            final_target: (target.x, target.y, 0.),
+            shot_vector: get_tuple_from_vec3(shot_vector),
+            final_target: get_tuple_from_vec3(flatten(target)),
             distance_remaining,
             path_samples,
+            required_jump_time,
+            current_path_point: get_tuple_from_vec3(flatten(current_path_point)),
         }
     }
 }
